﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace DependencyPropertySample
{
    public class Circle : DependencyObject
    {
        // Ereignis
        public event EventHandler RadiusChanged;

        // Eigenschaft
        public int Maximum { get; set; }

        // Dependency Property
        public static readonly DependencyProperty RadiusProperty;

        // Konstruktor
        public Circle(int maximum)
        {
            Maximum = maximum;
        }

        // statischer Konstruktor
        static Circle()
        {
            FrameworkPropertyMetadata meta = new FrameworkPropertyMetadata(0, FrameworkPropertyMetadataOptions.AffectsRender | FrameworkPropertyMetadataOptions.BindsTwoWayByDefault);
            meta.CoerceValueCallback = new CoerceValueCallback(CoerceRadius);

            meta.PropertyChangedCallback = new PropertyChangedCallback(OnRadiusChanged);
            RadiusProperty = DependencyProperty.Register("Radius", typeof(int), typeof(Circle), meta, new ValidateValueCallback(IsRadiusValid));
        }

        // bedient den delegaten 'CoerceValueCallback'
        private static object CoerceRadius(DependencyObject d, object value)
        {
            if (((Circle)d).Maximum >= (int)value)
                return value;
            return ((Circle)d).Maximum;
        }

        // bedient den Delegaten 'PropertyChangedCallback'
        public static void OnRadiusChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Circle kreis = (Circle)sender;
            if (kreis.RadiusChanged != null)
                kreis.RadiusChanged(kreis, null);
        }

        // Eigenschaftswrapper
        public int Radius
        {
            set { SetValue(RadiusProperty, value); }
            get { return (int)GetValue(RadiusProperty); }
        }

        // ValidateValueCallback
        private static bool IsRadiusValid(object value)
        {
            if ((int)value >= 0) return true;
            return false;
        }
    }
}
